//+------------------------------------------------------------------+
//|                                                 SampleSignal.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=  Blau_Ergodic                             |
//| Type=Signal                                                      |
//| Name=Blau_Ergodic                                                |
//| Class=CSampelSignal                                              |
//| Page=                                                            |
//| Parameter=q                                                      |
//| Parameter=r                                                      |
//| Parameter=s                                                      |
//| Parameter=u                                                      |
//| Parameter=ul                                                     |
//| Parameter=StopLoss,double,50.0                                   |
//| Parameter=TakeProfit,double,50.0                                 |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSampelSignal.                                             |
//| Purpose:                  |
//|           .                           |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CSampleSignal : public CExpertSignal
  {
protected:
   CiCustom           m_TSI;              //      TSI
   CiClose            m_close;           //       
  //---  
   int      q;  // q - ,    
   int      r; // r -  1- EMA,   
   int      s;  // s -  2- EMA,     
   int      u;  // u -  3- EMA,     
   int      ul; // ul - ( )  EMA,   
   double             m_stop_loss;       //    "stop loss"   
   double             m_take_profit;     //    "take profit"   
public:
              CSampleSignal();
 //---                  
   void               Q(int value)                 { q=value;   }
   void               R(int value)                 { r=value;   }
   void               S(int value)                 { s=value;   }
   void               U(int value)                 { u=value;   }
   void               UL(int value)                { ul=value;  }
   void               StopLoss(double value)       { m_stop_loss=value;   }
   void               TakeProfit(double value)     { m_take_profit=value; }
   //---    
   virtual bool       ValidationSettings();   
   virtual bool       InitIndicators(CIndicators* indicators);
    //---  ,     
   virtual bool      CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration);
protected:
   //---   
   bool               InitMyCustomIndicator(CIndicators* indicators);
   bool               InitClose(CIndicators* indicators);
   //---     
   double             Main(int index)                     { return(m_TSI.GetData(0,index)); }
   double             Signal(int index)                   { return(m_TSI.GetData(1,index)); }
   double             Signal_open_b(int index)           {return(Main(index)-Signal(index));}
   double             Signal_open_s(int index)           {return(Signal(index)-Main(index));}
   double             Close(int index)                    { return(m_close.GetData(index)); }
  };
//+------------------------------------------------------------------+
//|  CSampleSignal.                                       |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CSampleSignal::CSampleSignal()
  {
   q=2; 
   r=7;
   s=5;
   u=3;
   ul=5;   
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSampleSignal::~CSampleSignal(void)
  {
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::ValidationSettings()
  {
  //---  
   if(q<=0||r<=0||s<=0||u<=0||ul<=0)
     {
      printf(__FUNCTION__+":     ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitIndicators(CIndicators* indicators)
  {
//---  
   if(indicators==NULL)       return(false);
//---  TSI
   if(!InitMyCustomIndicator(indicators))    return(false);
   //---    
   if(!InitClose(indicators)) return(false);
   return(true);
  }
//+------------------------------------------------------------------+
//|  .                                       |
//+------------------------------------------------------------------+
bool CSampleSignal::InitMyCustomIndicator(CIndicators *indicators)
  {
//---    
   if(!indicators.Add(GetPointer(m_TSI)))
     {
      printf(__FUNCTION__+": error adding object");
      return(false);
     }
//---   
   MqlParam parameters[6];
//---
   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="Blau_Ergodic.ex5";
   parameters[1].type=TYPE_INT;
   parameters[1].integer_value=q;
   parameters[2].type=TYPE_INT;
   parameters[2].integer_value=r;
   parameters[3].type=TYPE_INT;
   parameters[3].integer_value=s;
   parameters[4].type=TYPE_INT;
   parameters[4].integer_value=u;
   parameters[5].type=TYPE_INT;
   parameters[5].integer_value=ul;
//---  
   if(!m_TSI.Create(m_symbol.Name(),0,IND_CUSTOM,6,parameters))
     {
      printf(__FUNCTION__+": error initializing object");
      return(false);
     }
//---  
   if(!m_TSI.NumBuffers(11)) return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                            |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitClose(CIndicators* indicators)
  {
//---   
   if(!m_close.Create(m_symbol.Name(),m_period))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---    
   if(!indicators.Add(GetPointer(m_close)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|     .                         |
//| INPUT:  price      -     ,        |
//|         sl         -     stop loss,       |
//|         tp         -     take profit,     |
//|         expiration -     .    |
//| OUTPUT: true-  ,  false.                |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration)
  {
//---  
   double buy    =Signal_open_b(1);
   double unit  =PriceLevelUnit();
//---  
   if(buy>1)
     {
      price=m_symbol.NormalizePrice(Close(0));
      sl   =m_symbol.NormalizePrice(price-m_stop_loss*unit);
      tp   =m_symbol.NormalizePrice(price+m_take_profit*unit);
      expiration+=m_expiration*PeriodSeconds(m_period);
      //---  
      return(true);
     }
//---   
   return(false);
  }